"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const boom_1 = require("boom");
const explore_container_1 = require("../usecases/explore-container");
const country_tags_service_1 = require("./country-tags-service");
const known_names_1 = require("@textactor/known-names");
class ContainerExplorer {
    constructor(containerId, options, containerRep, conceptRep, entityRep, searchNameRep, wikiTitleRep) {
        this.containerId = containerId;
        this.options = options;
        this.containerRep = containerRep;
        this.conceptRep = conceptRep;
        this.entityRep = entityRep;
        this.searchNameRep = searchNameRep;
        this.wikiTitleRep = wikiTitleRep;
        this.started = false;
        this.dataCallbacks = [];
        this.errorCallbacks = [];
        this.endCallbacks = [];
    }
    start() {
        if (this.started) {
            throw new Error(`Already started!`);
        }
        this.started = true;
        this.internalStart()
            .catch(error => this.emitError(error))
            .then(() => this.emitEnd());
        return this;
    }
    internalStart() {
        return __awaiter(this, void 0, void 0, function* () {
            const container = yield this.containerRep.getById(this.containerId);
            if (!container) {
                throw boom_1.notFound(`Not found container id=${this.containerId}`);
            }
            const processConcepts = new explore_container_1.ExploreContainer(container, this.containerRep, this.conceptRep, this.entityRep, this.searchNameRep, this.wikiTitleRep, new country_tags_service_1.CountryTagsService(), new known_names_1.KnownNameService());
            yield processConcepts.execute((actor) => this.emitData(actor), this.options);
        });
    }
    onData(callback) {
        this.dataCallbacks.push(callback);
    }
    emitData(data) {
        return __awaiter(this, void 0, void 0, function* () {
            yield Promise.all(this.dataCallbacks.map(callback => callback(data)));
        });
    }
    onError(callback) {
        this.errorCallbacks.push(callback);
    }
    emitError(error) {
        this.errorCallbacks.map(callback => callback(error));
    }
    onEnd(callback) {
        this.endCallbacks.push(callback);
    }
    emitEnd() {
        this.endCallbacks.map(callback => callback());
    }
}
exports.ContainerExplorer = ContainerExplorer;
