"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const concept_container_1 = require("../entities/concept-container");
const concept_container_helper_1 = require("../entities/concept-container-helper");
const concepts_parser_1 = require("concepts-parser");
const concept_helper_1 = require("../entities/concept-helper");
const push_context_concepts_1 = require("../usecases/actions/push-context-concepts");
const known_names_1 = require("@textactor/known-names");
function createDataContainerApi(containerRep, conceptRep) {
    const knownNames = new known_names_1.KnownNameService();
    const pushConcepts = new push_context_concepts_1.PushContextConcepts(conceptRep, knownNames);
    return {
        newDataContainer(data) {
            return __awaiter(this, void 0, void 0, function* () {
                const container = yield containerRep.create(concept_container_helper_1.ConceptContainerHelper.build(data));
                return {
                    container() { return container; },
                    pushText(text) {
                        return __awaiter(this, void 0, void 0, function* () {
                            if (container.status === concept_container_1.ConceptContainerStatus.NEW) {
                                yield containerRep.update({ id: container.id, set: { status: concept_container_1.ConceptContainerStatus.COLLECTING } });
                            }
                            const context = {
                                text,
                                lang: container.lang,
                                country: container.country,
                            };
                            const items = concepts_parser_1.parse(context, { mode: 'collect' });
                            if (!items || !items.length) {
                                return;
                            }
                            const concepts = items.map(item => {
                                const conceptData = {
                                    name: item.value, abbr: item.abbr, lang: context.lang,
                                    country: context.country,
                                    containerId: container.id,
                                };
                                return concept_helper_1.ConceptHelper.build(conceptData);
                            }).filter(item => concept_helper_1.ConceptHelper.isValid(item));
                            yield pushConcepts.execute(concepts);
                        });
                    },
                    pushTextNames(names) {
                        return __awaiter(this, void 0, void 0, function* () {
                            const concepts = names.map(name => {
                                const conceptData = {
                                    name, lang: container.lang,
                                    country: container.country,
                                    containerId: container.id,
                                };
                                return concept_helper_1.ConceptHelper.build(conceptData);
                            }).filter(item => concept_helper_1.ConceptHelper.isValid(item));
                            yield pushConcepts.execute(concepts);
                        });
                    },
                    end() {
                        return __awaiter(this, void 0, void 0, function* () {
                            container.status = concept_container_1.ConceptContainerStatus.COLLECT_DONE;
                            yield containerRep.update({ id: container.id, set: { status: concept_container_1.ConceptContainerStatus.COLLECT_DONE } });
                        });
                    }
                };
            });
        },
        findDataContainer(data) {
            return containerRep.list(data);
        }
    };
}
exports.createDataContainerApi = createDataContainerApi;
