"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mongo_repository_1 = require("./mongo/mongo-repository");
class ConceptContainerRepository extends mongo_repository_1.MongoRepository {
    getByStatus(locale, status) {
        return this.model.list({
            where: {
                lang: locale.lang,
                country: locale.country,
                status: { $in: status },
            },
            limit: 100,
        });
    }
    getByUniqueName(uniqueName) {
        return this.model.one({ where: { uniqueName } });
    }
    list(filters) {
        const selector = {
            where: {
                lang: filters.lang,
                country: filters.country,
            },
            limit: filters.limit,
            offset: filters.skip,
            sort: '-createdAt',
        };
        if (filters.ownerId) {
            selector.where.ownerId = filters.ownerId;
        }
        if (filters.uniqueName) {
            selector.where.uniqueName = filters.uniqueName;
        }
        if (filters.status) {
            selector.where.status = { $in: filters.status };
        }
        return this.model.list(selector);
    }
    count(locale) {
        return this.model.count({
            lang: locale.lang,
            count: locale.country,
        });
    }
    deleteIds(ids) {
        return this.model.remove({
            where: {
                _id: { $in: ids }
            }
        });
    }
    createOrUpdate(item) {
        return this.create(item).catch(error => {
            if (error.code && error.code === 11000) {
                return this.getById(item.id).then(dbItem => {
                    if (dbItem) {
                        return this.update({ id: item.id, set: item });
                    }
                    else {
                        throw new Error(`!NOT found concept on updating: ${item.name}`);
                    }
                });
            }
            return Promise.reject(error);
        });
    }
}
exports.ConceptContainerRepository = ConceptContainerRepository;
