"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mongo_repository_1 = require("./mongo/mongo-repository");
const utils_1 = require("../utils");
class ConceptRepository extends mongo_repository_1.MongoRepository {
    getMostPopular(containerId, limit, skip, options) {
        options = options || {};
        const where = { containerId };
        if (options.minCountWords) {
            where.countWords = where.countWords || {};
            where.countWords.$gte = options.minCountWords;
        }
        if (options.maxCountWords) {
            where.countWords = where.countWords || {};
            where.countWords.$lte = options.maxCountWords;
        }
        return this.model.list({
            where,
            limit,
            offset: skip,
            sort: '-popularity,createdAt',
        });
    }
    getByRootNameId(id) {
        return this.model.list({
            where: {
                rootNameIds: id
            },
            limit: 500,
        });
    }
    getByRootNameIds(ids) {
        return this.model.list({
            where: {
                rootNameIds: { $in: ids }
            },
            limit: 500,
        });
    }
    list(locale, limit, skip) {
        return this.model.list({
            where: {
                lang: locale.lang,
                country: locale.country,
            },
            limit,
            offset: skip || 0,
        });
    }
    getConceptsWithAbbr(containerId) {
        return this.model.list({
            where: {
                containerId,
                abbr: { $exists: true },
                limit: 500,
            }
        });
    }
    count(locale) {
        return this.model.count({
            lang: locale.lang,
            count: locale.country,
        });
    }
    deleteUnpopular(containerId, popularity) {
        return this.model.remove({
            where: {
                containerId,
                popularity: { $lt: popularity },
            }
        });
    }
    deleteUnpopularAbbreviations(containerId, popularity) {
        return this.model.remove({
            where: {
                containerId,
                isAbbr: true,
                popularity: { $lt: popularity },
            }
        });
    }
    deleteUnpopularOneWorlds(containerId, popularity) {
        return this.model.remove({
            where: {
                containerId,
                countWords: 1,
                isAbbr: false,
                popularity: { $lt: popularity },
            }
        });
    }
    deleteAll(containerId) {
        return this.model.remove({
            where: { containerId }
        });
    }
    deleteIds(ids) {
        return this.model.remove({
            where: {
                _id: { $in: ids }
            }
        });
    }
    deleteByRootNameIds(ids) {
        return this.model.remove({
            where: {
                rootNameIds: { $in: ids }
            }
        });
    }
    createOrUpdate(item) {
        return this.create(item).catch(error => {
            if (error.code && error.code === 11000) {
                return this.getById(item.id).then(dbItem => {
                    if (dbItem) {
                        item.popularity = dbItem.popularity + 1;
                        if (item.rootNameIds) {
                            item.rootNameIds = utils_1.uniq(dbItem.rootNameIds.concat(item.rootNameIds));
                        }
                        return this.update({ id: item.id, set: item });
                    }
                    else {
                        throw new Error(`!NOT found concept on updating: ${item.name}`);
                    }
                });
            }
            return Promise.reject(error);
        });
    }
}
exports.ConceptRepository = ConceptRepository;
