"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mongoose_1 = require("mongoose");
const utils_1 = require("../../utils");
const mongo_model_1 = require("./mongo-model");
class ConceptContainerModel extends mongo_model_1.MongoModel {
    constructor(connection) {
        super(connection.model('ConceptContainer', ModelSchema));
    }
    transformItem(item) {
        const data = super.transformItem(item);
        if (data) {
            if (data.createdAt) {
                data.createdAt = Math.round(new Date(data.createdAt).getTime() / 1000);
            }
        }
        return data;
    }
    beforeCreating(data) {
        data.createdAt = data.createdAt || utils_1.unixTimestamp();
        data.updatedAt = data.updatedAt || data.createdAt;
        data.createdAt = new Date(data.createdAt * 1000);
        data.updatedAt = new Date(data.updatedAt * 1000);
        return super.beforeCreating(data);
    }
    beforeUpdating(data) {
        if (data.set) {
            delete data.set.createdAt;
            data.set.updatedAt = data.set.updatedAt || utils_1.unixTimestamp();
            if (typeof data.set.updatedAt === 'number') {
                data.set.updatedAt = new Date(data.set.updatedAt * 1000);
            }
        }
        return super.beforeUpdating(data);
    }
}
exports.ConceptContainerModel = ConceptContainerModel;
const ModelSchema = new mongoose_1.Schema({
    _id: String,
    lang: {
        type: String,
        match: utils_1.LANG_REG,
        required: true,
        index: true,
    },
    country: {
        type: String,
        match: utils_1.COUNTRY_REG,
        required: true,
        index: true,
    },
    name: {
        type: String,
        minlength: 2,
        maxlength: 200,
        required: true,
    },
    uniqueName: {
        type: String,
        minlength: 2,
        maxlength: 200,
        required: true,
        unique: true,
    },
    status: {
        type: String,
        minlength: 1,
        maxlength: 50,
        required: true,
        index: true,
    },
    ownerId: {
        type: String,
        minlength: 1,
        maxlength: 50,
        index: true,
        required: true,
    },
    lastError: {
        type: String,
        maxlength: 800,
    },
    createdAt: {
        type: Date,
        default: Date.now,
        required: true,
        expires: '15 days',
    },
    updatedAt: {
        type: Date,
        default: Date.now,
        required: true,
    },
}, {
    collection: 'textactor_conceptContainers'
});
ModelSchema.set('toObject', {
    getters: true
});
