"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const boom_1 = require("boom");
class MongoModel {
    constructor(model) {
        this.model = model;
    }
    create(data) {
        if (!data) {
            return Promise.reject(Error('`data` is required'));
        }
        try {
            data = this.beforeCreating(data);
        }
        catch (e) {
            return Promise.reject(e);
        }
        return new Promise((resolve, reject) => {
            this.model.create(data).then(item => resolve(item && this.transformItem(item)), reject);
        });
    }
    update(data) {
        return __awaiter(this, void 0, void 0, function* () {
            data = this.beforeUpdating(data);
            const updateData = {};
            if (data.set && Object.keys(data.set).length) {
                updateData['$set'] = data.set;
            }
            if (data.unset && Object.keys(data.unset).length) {
                updateData['$unset'] = data.unset;
            }
            const document = yield this.model.findByIdAndUpdate(data.id, updateData);
            if (document) {
                return this.transformItem(document);
            }
            else {
                throw boom_1.notFound(`Not found object id=${data.id}`, data);
            }
        });
    }
    remove(params) {
        if (!params) {
            return Promise.reject(Error('`params` is required'));
        }
        return new Promise((resolve, reject) => {
            this.model.remove(params.where)
                .then(item => resolve(item), reject);
        });
    }
    one(params) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!params) {
                return Promise.reject(Error('`params` is required'));
            }
            const document = yield this.model.findOne(params.where, params.select);
            if (document) {
                return this.transformItem(document);
            }
            else {
                return null;
            }
        });
    }
    count(where) {
        return new Promise((resolve, reject) => {
            this.model.count(where).then(item => resolve(item), reject);
        });
    }
    list(params) {
        if (!params) {
            return Promise.reject(Error('`params` is required'));
        }
        return new Promise((resolve, reject) => {
            this.model
                .find(params.where)
                .select(params.select)
                .sort(params.sort)
                .skip(params.offset || 0)
                .limit(params.limit || 10)
                .exec()
                .then(items => resolve(items && items.map(item => this.transformItem(item))), reject);
        });
    }
    beforeCreating(data) {
        const ndata = data;
        for (let prop of Object.keys(ndata)) {
            if (~[null, undefined].indexOf(ndata[prop])) {
                delete ndata[prop];
            }
        }
        ndata._id = ndata._id || ndata.id;
        return data;
    }
    beforeUpdating(data) {
        return data;
    }
    transformItem(item) {
        return item.toJSON();
    }
}
exports.MongoModel = MongoModel;
