"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class MongoRepository {
    constructor(model) {
        this.model = model;
    }
    getById(id) {
        return this.model.one({ where: { _id: id } });
    }
    getByIds(ids) {
        return this.model.list({ where: { _id: { $in: ids } }, limit: ids.length });
    }
    exists(id) {
        return this.model.one({ where: { _id: id }, select: '_id' }).then(item => !!item);
    }
    delete(id) {
        return this.model.remove({ where: { _id: id } }).then(item => !!item);
    }
    create(data) {
        return this.model.create(data);
    }
    update(data) {
        const id = data.id;
        const set = {};
        if (data.set) {
            for (let prop of Object.keys(data.set)) {
                if (['id', 'createdAt', '_id'].indexOf(prop) > -1) {
                    continue;
                }
                if ([null, undefined, ''].indexOf(data.set[prop]) > -1) {
                    continue;
                }
                set[prop] = data.set[prop];
            }
        }
        const unset = {};
        if (data.delete) {
            for (let prop of data.delete) {
                if (['id', 'createdAt', '_id'].indexOf(prop) > -1) {
                    continue;
                }
                unset[prop] = "";
            }
        }
        return this.model.update({
            id: id,
            set: set,
            unset
        });
    }
    createOrUpdate(item) {
        return this.create(item).catch(error => {
            if (error.code && error.code === 11000) {
                return this.update({ id: item.id, set: item });
            }
            return Promise.reject(error);
        });
    }
}
exports.MongoRepository = MongoRepository;
