"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mongoose_1 = require("mongoose");
const utils_1 = require("../../utils");
const mongo_model_1 = require("./mongo-model");
class WikiEntityModel extends mongo_model_1.MongoModel {
    constructor(connection) {
        super(connection.model('WikiEntity', ModelSchema));
    }
    transformItem(item) {
        const data = super.transformItem(item);
        if (data) {
            if (data.createdAt) {
                data.createdAt = Math.round(new Date(data.createdAt).getTime() / 1000);
            }
        }
        return data;
    }
    beforeCreating(data) {
        data.createdAt = data.createdAt || utils_1.unixTimestamp();
        data.updatedAt = data.updatedAt || data.createdAt;
        data.createdAt = new Date(data.createdAt * 1000);
        data.updatedAt = new Date(data.updatedAt * 1000);
        return super.beforeCreating(data);
    }
    beforeUpdating(data) {
        if (data.set) {
            delete data.set.createdAt;
            data.set.updatedAt = data.set.updatedAt || utils_1.unixTimestamp();
            if (typeof data.set.updatedAt === 'number') {
                data.set.updatedAt = new Date(data.set.updatedAt * 1000);
            }
        }
        return super.beforeUpdating(data);
    }
}
exports.WikiEntityModel = WikiEntityModel;
const ModelSchema = new mongoose_1.Schema({
    _id: String,
    lang: {
        type: String,
        match: utils_1.LANG_REG,
        required: true,
        index: true,
    },
    name: {
        type: String,
        minlength: 2,
        maxlength: 200,
        required: true,
    },
    simpleName: {
        type: String,
        minlength: 2,
        maxlength: 200,
    },
    specialName: {
        type: String,
        minlength: 1,
        maxlength: 200,
    },
    names: {
        type: [String],
        required: true,
    },
    namesHashes: {
        type: [String],
        index: true,
        required: true,
    },
    partialNames: {
        type: [String],
        required: true,
    },
    partialNamesHashes: {
        type: [String],
        index: true,
        required: true,
    },
    abbr: {
        type: String,
    },
    description: {
        type: String
    },
    aliases: {
        type: [String]
    },
    about: {
        type: String
    },
    wikiDataId: {
        type: String
    },
    wikiPageId: {
        type: Number
    },
    wikiPageTitle: {
        type: String
    },
    type: {
        type: String
    },
    types: {
        type: [String]
    },
    countryCodes: {
        type: [String]
    },
    rank: {
        type: Number
    },
    categories: {
        type: [String]
    },
    data: {
        type: mongoose_1.Schema.Types.Mixed
    },
    links: {
        type: mongoose_1.Schema.Types.Mixed
    },
    createdAt: {
        type: Date,
        default: Date.now,
        required: true,
    },
    updatedAt: {
        type: Date,
        default: Date.now,
        required: true,
        expires: '15 days',
    },
}, {
    collection: 'textactor_wikiEntities'
});
ModelSchema.set('toObject', {
    getters: true
});
