"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const mongo_repository_1 = require("./mongo/mongo-repository");
const wiki_entity_1 = require("../entities/wiki-entity");
const name_helper_1 = require("../name-helper");
class WikiEntityRepository extends mongo_repository_1.MongoRepository {
    getByNameHash(hash) {
        return this.model.list({
            where: { namesHashes: hash },
            limit: 100,
        });
    }
    getByPartialNameHash(hash) {
        return this.model.list({
            where: { partialNamesHashes: hash },
            limit: 100,
        });
    }
    count() {
        throw new Error("Method not implemented.");
    }
    getInvalidPartialNames(lang) {
        return __awaiter(this, void 0, void 0, function* () {
            const container = {};
            function processList(list) {
                for (let item of list) {
                    item.names.forEach(name => {
                        if (name_helper_1.NameHelper.countWords(name) < 2 || name_helper_1.NameHelper.isAbbr(name)) {
                            return;
                        }
                        const parts = name.split(/\s+/g).filter(it => !name_helper_1.NameHelper.isAbbr(it) && it.length > 1 && it[0] !== '(' && it.toLowerCase() !== it);
                        for (let it of parts) {
                            container[it] = true;
                        }
                    });
                }
            }
            let list = yield this.model.list({
                where: {
                    lang: lang,
                    type: wiki_entity_1.WikiEntityType.PERSON,
                },
                limit: 500
            });
            processList(list);
            if (list.length === 500) {
                list = yield this.model.list({
                    where: {
                        lang: lang,
                        type: wiki_entity_1.WikiEntityType.PERSON,
                    },
                    limit: 500
                });
                processList(list);
            }
            return Object.keys(container);
        });
    }
}
exports.WikiEntityRepository = WikiEntityRepository;
