"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const name_helper_1 = require("../name-helper");
class ActorNameCollection {
    constructor(lang) {
        this.lang = lang;
        this.map = {};
    }
    get length() {
        return Object.keys(this.map).length;
    }
    add({ name, popularity, type }) {
        name = name.trim();
        const id = name_helper_1.NameHelper.normalizeName(name, this.lang);
        if (name.length < 2 || id.length < 2) {
            return this;
        }
        const item = this.map[id];
        if (item) {
            if (item.popularity < popularity) {
                item.popularity = popularity;
            }
            if (item.type === 'SAME' && type === 'WIKI') {
                item.type = type;
            }
        }
        else {
            const isAbbr = name_helper_1.NameHelper.isAbbr(name);
            this.map[id] = { name, popularity, id, type, isAbbr, order: this.length };
        }
        return this;
    }
    list() {
        return Object.keys(this.map)
            .reduce((list, id) => {
            list.push(this.map[id]);
            return list;
        }, [])
            .sort((a, b) => {
            const d = b.popularity - a.popularity;
            if (d === 0) {
                return a.order - b.order;
            }
            return d;
        });
    }
    nameList() {
        return this.list().map(item => item.name);
    }
    static fromConcepts(lang, concepts) {
        const collection = new ActorNameCollection(lang);
        for (const item of concepts) {
            if (item.knownName) {
                collection.add({ name: item.knownName, type: 'SAME', popularity: item.popularity });
            }
            collection.add({ name: item.name, type: 'SAME', popularity: item.popularity });
        }
        return collection;
    }
    static fromArray(names, lang, type = 'SAME', popularity = 1) {
        const collection = new ActorNameCollection(lang);
        for (const name of names) {
            collection.add({ name, popularity, type });
        }
        return collection;
    }
    addArray(names, type = 'SAME', popularity = 1) {
        for (const name of names) {
            this.add({ name, popularity, type });
        }
        return this;
    }
}
exports.ActorNameCollection = ActorNameCollection;
