"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const concept_container_1 = require("./concept-container");
const shortid_1 = require("shortid");
class ConceptContainerHelper {
    static build(data) {
        const container = {
            id: shortid_1.generate(),
            status: concept_container_1.ConceptContainerStatus.NEW,
            name: data.name,
            uniqueName: data.uniqueName,
            lang: data.lang,
            country: data.country,
            ownerId: data.ownerId,
        };
        return container;
    }
    static newId() {
        return shortid_1.generate();
    }
    static validate(entity) {
        if (!entity) {
            throw new Error(`Invalid ConceptContainer: null or undefined`);
        }
        if (!entity.id || entity.id.trim().length < 2) {
            throw new Error(`Invalid ConceptContainer: invalid id`);
        }
        if (!entity.lang) {
            throw new Error(`Invalid ConceptContainer: invalid lang`);
        }
        if (!entity.country) {
            throw new Error(`Invalid ConceptContainer: invalid country`);
        }
        if (!entity.name) {
            throw new Error(`Invalid ConceptContainer: invalid name`);
        }
        if (!entity.uniqueName) {
            throw new Error(`Invalid ConceptContainer: invalid uniqueName`);
        }
        if (!entity.ownerId) {
            throw new Error(`Invalid ConceptContainer: invalid ownerId`);
        }
        if (!entity.status) {
            throw new Error(`Invalid ConceptContainer: invalid status`);
        }
    }
    static canStartCollect(status) {
        return [
            concept_container_1.ConceptContainerStatus.NEW
        ].indexOf(status) > -1;
    }
    static canStartGenerate(status) {
        return [
            concept_container_1.ConceptContainerStatus.COLLECT_DONE,
            concept_container_1.ConceptContainerStatus.COLLECT_ERROR,
            concept_container_1.ConceptContainerStatus.GENERATE_ERROR,
        ].indexOf(status) > -1;
    }
}
exports.ConceptContainerHelper = ConceptContainerHelper;
