"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const name_helper_1 = require("../name-helper");
const utils_1 = require("../utils");
const actor_name_collection_1 = require("./actor-name-collection");
class ConceptHelper {
    static build(data) {
        const lang = data.lang.trim().toLowerCase();
        const country = data.country.trim().toLowerCase();
        const containerId = data.containerId.trim();
        const name = data.name.trim();
        const nameLength = name.length;
        const normalName = name_helper_1.NameHelper.normalizeName(name, lang);
        const id = ConceptHelper.id(normalName, lang, country, containerId);
        const isAbbr = name_helper_1.NameHelper.isAbbr(name);
        const countWords = name_helper_1.NameHelper.countWords(name);
        const isIrregular = name_helper_1.NameHelper.isIrregular(name);
        const endsWithNumber = name_helper_1.NameHelper.endsWithNumberWord(name);
        const rootNameIds = ConceptHelper.rootIds([data.knownName, name], lang, country, containerId);
        const popularity = data.popularity || 1;
        const concept = {
            id,
            country,
            lang,
            containerId,
            name,
            nameLength,
            isAbbr,
            countWords,
            isIrregular,
            endsWithNumber,
            abbr: data.abbr,
            rootNameIds,
            popularity,
            context: data.context,
        };
        if (data.knownName && ConceptHelper.isValidName(data.knownName, lang)) {
            concept.knownName = data.knownName.trim();
        }
        return concept;
    }
    static nameHash(name, lang, country, containerId) {
        name = name.trim();
        name = name_helper_1.NameHelper.normalizeName(name, lang);
        name = name_helper_1.NameHelper.atonic(name);
        return ConceptHelper.hash(name, lang, country, containerId);
    }
    static hash(name, lang, country, containerId) {
        return utils_1.md5([lang.trim().toLowerCase(), country.trim().toLowerCase(), containerId.trim(), name.trim()].join('_'));
    }
    static id(name, lang, country, containerId) {
        name = name_helper_1.NameHelper.normalizeName(name, lang);
        return ConceptHelper.hash(name, lang, country, containerId);
    }
    static ids(names, lang, country, containerId) {
        return utils_1.uniq(names.map(name => ConceptHelper.id(name, lang, country, containerId)));
    }
    static rootName(name, lang) {
        lang = lang.trim();
        name = name.trim();
        name = name_helper_1.NameHelper.normalizeName(name, lang);
        name = name_helper_1.NameHelper.atonic(name);
        return name;
    }
    static rootId(name, lang, country, containerId) {
        name = ConceptHelper.rootName(name, lang);
        return ConceptHelper.id(name, lang, country, containerId);
    }
    static rootIds(names, lang, country, containerId) {
        const filteredNames = names.filter(name => name && name.trim().length > 1);
        return utils_1.uniq(filteredNames.map(name => ConceptHelper.rootId(name, lang, country, containerId)));
    }
    static setRootIds(concept) {
        concept.rootNameIds = utils_1.uniq(concept.rootNameIds.concat(ConceptHelper.rootIds([concept.knownName, concept.name], concept.lang, concept.country, concept.containerId)));
    }
    static getConceptsNames(concepts) {
        if (concepts.length === 0) {
            throw new Error(`No concepts!`);
        }
        const { lang } = concepts[0];
        concepts = concepts.sort((a, b) => b.popularity - a.popularity);
        return actor_name_collection_1.ActorNameCollection.fromConcepts(lang, concepts);
    }
    static isValidName(name, lang) {
        return typeof name === 'string' && name.trim().length > 1 && name_helper_1.NameHelper.normalizeName(name, lang).length > 1;
    }
    static isValid(concept) {
        return ConceptHelper.isValidName(concept.name, concept.lang);
    }
}
exports.ConceptHelper = ConceptHelper;
