"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
class MemoryConceptContainerRepository {
    constructor() {
        this.db = new Map();
    }
    count(locale) {
        let count = 0;
        for (let item of this.db.values()) {
            if (item.country !== locale.country || item.lang !== locale.lang) {
                continue;
            }
            count++;
        }
        return Promise.resolve(count);
    }
    getByStatus(locale, status) {
        const list = [];
        for (let item of this.db.values()) {
            if (item.country !== locale.country || item.lang !== locale.lang || status.indexOf(item.status) < 0) {
                continue;
            }
            list.push(item);
        }
        return Promise.resolve(list);
    }
    getByUniqueName(uniqueName) {
        for (let item of this.db.values()) {
            if (item.uniqueName === uniqueName) {
                return Promise.resolve(item);
            }
        }
        return Promise.resolve(null);
    }
    list(filters) {
        const skip = filters.skip || 0;
        const list = [];
        for (let item of this.db.values()) {
            if (item.country !== filters.country || item.lang !== filters.lang) {
                continue;
            }
            if (filters.ownerId && filters.ownerId !== item.ownerId) {
                continue;
            }
            if (filters.uniqueName && filters.uniqueName !== item.uniqueName) {
                continue;
            }
            if (filters.status && filters.status.indexOf(item.status) < 0) {
                continue;
            }
            list.push(item);
        }
        return Promise.resolve(list.slice(skip, skip + filters.limit));
    }
    getById(id) {
        return Promise.resolve(this.db.get(id) || null);
    }
    getByIds(ids) {
        const list = [];
        for (let id of ids) {
            const item = this.db.get(id);
            if (item) {
                list.push(item);
            }
        }
        return Promise.resolve(list);
    }
    exists(id) {
        return Promise.resolve(!!this.db.get(id));
    }
    delete(id) {
        return Promise.resolve(this.db.delete(id));
    }
    create(data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!!this.db.get(data.id)) {
                return Promise.reject(new Error(`Item already exists!`));
            }
            this.db.set(data.id, Object.assign({ createdAt: Date.now() }, data));
            const entity = yield this.getById(data.id);
            if (!entity) {
                throw new Error(`Entity not found!`);
            }
            return entity;
        });
    }
    update(data) {
        const item = this.db.get(data.id);
        if (!item) {
            return Promise.reject(new Error(`Item not found! id=${data.id}`));
        }
        if (!data.set && !data.delete) {
            return Promise.reject(new Error(`'set' or 'delete' must exist`));
        }
        if (data.set) {
            for (let prop in data.set) {
                if ([null, undefined].indexOf(data.set[prop]) < 0) {
                    item[prop] = data.set[prop];
                }
            }
        }
        if (data.delete) {
            for (let prop of data.delete) {
                delete item[prop];
            }
        }
        return Promise.resolve(item);
    }
    deleteIds(ids) {
        let count = 0;
        for (let id of ids) {
            this.db.delete(id) && count++;
        }
        return Promise.resolve(count);
    }
    all() {
        const array = [];
        for (let item of this.db.values()) {
            array.push(item);
        }
        return Promise.resolve(array);
    }
}
exports.MemoryConceptContainerRepository = MemoryConceptContainerRepository;
