"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const wiki_entity_1 = require("../../entities/wiki-entity");
const name_helper_1 = require("../../name-helper");
const utils_1 = require("../../utils");
class MemoryWikiEntityRepository {
    constructor() {
        this.db = new Map();
    }
    count() {
        return Promise.resolve(this.db.size);
    }
    getInvalidPartialNames(lang) {
        const container = {};
        for (let item of this.db.values()) {
            if (item.lang === lang && item.type === wiki_entity_1.WikiEntityType.PERSON) {
                item.names.forEach(name => {
                    if (name_helper_1.NameHelper.countWords(name) < 2 || name_helper_1.NameHelper.isAbbr(name)) {
                        return;
                    }
                    const parts = name.split(/\s+/g).filter(it => !name_helper_1.NameHelper.isAbbr(it) && it.length > 1);
                    for (let it of parts) {
                        container[it] = true;
                    }
                });
            }
        }
        return Promise.resolve(Object.keys(container));
    }
    getByPartialNameHash(hash) {
        const list = [];
        for (let item of this.db.values()) {
            if (item.partialNamesHashes && ~item.partialNamesHashes.indexOf(hash)) {
                list.push(item);
            }
        }
        return Promise.resolve(utils_1.uniq(list));
    }
    getByNameHash(hash) {
        const list = [];
        for (let item of this.db.values()) {
            if (~item.namesHashes.indexOf(hash)) {
                list.push(item);
            }
        }
        return Promise.resolve(utils_1.uniq(list));
    }
    getById(id) {
        return Promise.resolve(this.db.get(id) || null);
    }
    getByIds(ids) {
        const list = [];
        for (let id of ids) {
            const item = this.db.get(id);
            if (item) {
                list.push(item);
            }
        }
        return Promise.resolve(list);
    }
    exists(id) {
        return Promise.resolve(!!this.db.get(id));
    }
    delete(id) {
        return Promise.resolve(this.db.delete(id));
    }
    create(data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!!this.db.get(data.id)) {
                return Promise.reject(new Error(`Item already exists!`));
            }
            this.db.set(data.id, Object.assign({ createdAt: new Date() }, data));
            const entity = yield this.getById(data.id);
            if (!entity) {
                throw new Error(`Entity not found!`);
            }
            return entity;
        });
    }
    update(data) {
        const item = this.db.get(data.id);
        if (!item) {
            return Promise.reject(new Error(`Item not found! id=${data.id}`));
        }
        if (data.set) {
            for (let prop in data.set) {
                item[prop] = data.set[prop];
            }
        }
        if (data.delete) {
            for (let prop of data.delete) {
                delete item[prop];
            }
        }
        return Promise.resolve(item);
    }
    createOrUpdate(data) {
        if (!!this.db.get(data.id)) {
            return this.update({ id: data.id, set: data });
        }
        return this.create(data);
    }
}
exports.MemoryWikiEntityRepository = MemoryWikiEntityRepository;
