"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
class MemoryWikiSearchNameRepository {
    constructor() {
        this.db = new Map();
    }
    createOrUpdate(data) {
        const item = this.db.get(data.id);
        if (item) {
            return this.update({ id: data.id, set: data });
        }
        return this.create(data);
    }
    getById(id) {
        return Promise.resolve(this.db.get(id) || null);
    }
    getByIds(ids) {
        const list = [];
        for (let id of ids) {
            const item = this.db.get(id);
            if (item) {
                list.push(item);
            }
        }
        return Promise.resolve(list);
    }
    exists(id) {
        return Promise.resolve(!!this.db.get(id));
    }
    delete(id) {
        return Promise.resolve(this.db.delete(id));
    }
    create(data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!!this.db.get(data.id)) {
                return Promise.reject(new Error(`Item already exists!`));
            }
            this.db.set(data.id, Object.assign({ createdAt: new Date(), lastSearchAt: new Date() }, data));
            const entity = yield this.getById(data.id);
            if (!entity) {
                throw new Error(`Entity not found!`);
            }
            return entity;
        });
    }
    update(data) {
        const item = this.db.get(data.id);
        if (!item) {
            return Promise.reject(new Error(`Item not found! id=${data.id}`));
        }
        if (data.set) {
            delete data.set.createdAt;
            for (let prop in data.set) {
                item[prop] = data.set[prop];
            }
        }
        if (data.delete) {
            for (let prop of data.delete) {
                delete item[prop];
            }
        }
        return Promise.resolve(item);
    }
}
exports.MemoryWikiSearchNameRepository = MemoryWikiSearchNameRepository;
