"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug = require('debug')('textactor-explorer');
const concept_helper_1 = require("../entities/concept-helper");
const utils_1 = require("../utils");
const actor_name_collection_1 = require("../entities/actor-name-collection");
const START_MIN_COUNT_WORDS = 2;
class PopularConceptNamesEnumerator {
    constructor(options, container, conceptRep) {
        this.options = options;
        this.container = container;
        this.conceptRep = conceptRep;
        this.skip = 0;
        this.limit = 10;
        this.currentIndex = -1;
        this.currentData = null;
        this.end = false;
        this.minCountWords = START_MIN_COUNT_WORDS;
        this.pagesize = 10;
        if (options.mutable) {
            this.limit = this.pagesize = 1;
        }
    }
    reset() {
        this.skip = 0;
        this.limit = this.pagesize;
        this.currentIndex = -1;
        this.currentData = null;
    }
    atEnd() {
        return this.end;
    }
    next() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.end) {
                debug(`END`);
                return new actor_name_collection_1.ActorNameCollection(this.container.lang);
            }
            if (this.currentData && this.currentIndex < this.currentData.length) {
                return yield this.getConceptNames(this.currentData[this.currentIndex++]);
            }
            const concepts = yield this.conceptRep.getMostPopular(this.container.id, this.limit, this.skip, { minCountWords: this.minCountWords, maxCountWords: this.maxCountWords });
            if (concepts.length === 0) {
                if (this.minCountWords === START_MIN_COUNT_WORDS) {
                    this.minCountWords = 0;
                    this.maxCountWords = 1;
                    this.reset();
                    return this.next();
                }
                this.end = true;
                return new actor_name_collection_1.ActorNameCollection(this.container.lang);
            }
            if (this.options.mutable) {
                if (this.currentData && this.currentData.length && concepts.length && this.currentData[0].id === concepts[0].id) {
                    throw new Error(`Got the same concept: ${concepts[0].name}`);
                }
            }
            else {
                this.skip += this.limit;
            }
            this.currentData = concepts;
            this.currentIndex = 0;
            return yield this.getConceptNames(this.currentData[this.currentIndex++]);
        });
    }
    getConceptNames(concept) {
        return __awaiter(this, void 0, void 0, function* () {
            let concepts = yield this.conceptRep.getByRootNameIds(concept.rootNameIds);
            const rootIds = utils_1.uniq(concepts.reduce((ids, current) => ids.concat(current.rootNameIds), []));
            if (rootIds.length > concept.rootNameIds.length) {
                concepts = utils_1.uniqByProp(yield this.conceptRep.getByRootNameIds(rootIds), 'id');
                debug(`Found more concepts by ${concept.name}: ${JSON.stringify(concepts.map(item => item.name))}`);
            }
            const names = concept_helper_1.ConceptHelper.getConceptsNames([concept].concat(concepts));
            return names;
        });
    }
}
exports.PopularConceptNamesEnumerator = PopularConceptNamesEnumerator;
