"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug = require('debug')('textactor-explorer');
const find_wiki_entities_by_titles_1 = require("./find-wiki-entities-by-titles");
const save_wiki_entities_1 = require("./save-wiki-entities");
const find_wiki_titles_1 = require("./find-wiki-titles");
const wiki_search_name_1 = require("../../entities/wiki-search-name");
const wiki_title_1 = require("../../entities/wiki-title");
const usecase_1 = require("../usecase");
const ms = require("ms");
const utils_1 = require("../../utils");
class ExploreWikiEntitiesByNames extends usecase_1.UseCase {
    constructor(locale, entityRep, wikiSearchNameRep, wikiTitleRep, countryTags, knownNames) {
        super();
        this.locale = locale;
        this.wikiSearchNameRep = wikiSearchNameRep;
        this.wikiTitleRep = wikiTitleRep;
        this.exploreWikiEntitiesByTitles = new find_wiki_entities_by_titles_1.FindWikiEntitiesByTitles(locale, knownNames);
        this.saveWikiEntities = new save_wiki_entities_1.SaveWikiEntities(entityRep);
        this.findWikiTitles = new find_wiki_titles_1.FindWikiTitles(locale, countryTags);
    }
    innerExecute(names) {
        return __awaiter(this, void 0, void 0, function* () {
            const lang = this.locale.lang;
            const country = this.locale.country;
            names = utils_1.uniq(names);
            const unknownNames = [];
            for (let name of names) {
                const searchName = yield this.wikiSearchNameRep.getById(wiki_search_name_1.WikiSearchNameHelper.createId(name, lang, country));
                if (searchName && searchName.updatedAt && searchName.updatedAt * 1000 > Date.now() - ms('7days')) {
                    debug(`WikiSearchName=${name} exists!`);
                    continue;
                }
                unknownNames.push(name);
                yield this.wikiSearchNameRep.createOrUpdate(wiki_search_name_1.WikiSearchNameHelper.build({
                    name,
                    lang,
                    country,
                }));
            }
            if (unknownNames.length === 0) {
                return [];
            }
            const initalTitles = yield this.findWikiTitles.execute(unknownNames);
            if (!initalTitles.length) {
                return [];
            }
            const titles = [];
            for (let title of initalTitles) {
                const wikiTitle = yield this.wikiTitleRep.getById(wiki_title_1.WikiTitleHelper.createId(title, lang));
                if (wikiTitle && wikiTitle.updatedAt && wikiTitle.updatedAt * 1000 > Date.now() - ms('10days')) {
                    debug(`WikiTitle=${title} exists!`);
                    continue;
                }
                titles.push(title);
            }
            if (titles.length === 0) {
                return [];
            }
            const wikiEntities = yield this.exploreWikiEntitiesByTitles.execute(titles);
            if (wikiEntities.length) {
                debug(`found wiki entities for ${names[0]}==${wikiEntities.length}`);
                yield this.saveWikiEntities.execute(wikiEntities);
            }
            else {
                debug(`Not found wiki entities for ${names[0]}`);
            }
            for (let title of titles) {
                yield this.wikiTitleRep.createOrUpdate(wiki_title_1.WikiTitleHelper.create({ title, lang, }));
            }
            return titles;
        });
    }
}
exports.ExploreWikiEntitiesByNames = ExploreWikiEntitiesByNames;
