"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug = require('debug')('textactor-explorer');
const wiki_entity_1 = require("wiki-entity");
const wiki_entity_helper_1 = require("../../entities/wiki-entity-helper");
const utils_1 = require("../../utils");
const wiki_entity_builder_1 = require("./wiki-entity-builder");
const usecase_1 = require("../usecase");
class FindWikiEntitiesByTitles extends usecase_1.UseCase {
    constructor(locale, knownNames) {
        super();
        this.locale = locale;
        this.wikiEntityBuilder = new wiki_entity_builder_1.WikiEntityBuilder(locale, knownNames);
    }
    innerExecute(titles) {
        return __awaiter(this, void 0, void 0, function* () {
            titles = utils_1.uniq(titles);
            debug(`exploring wiki entities for ${titles.join('|')}`);
            const findOptions = {
                titles: titles,
                claims: 'item',
                categories: true,
                extract: 3,
                language: this.locale.lang,
                redirects: true,
                types: true,
            };
            let wikiEntities = [];
            try {
                wikiEntities = yield wiki_entity_1.getEntities(findOptions);
            }
            catch (e) {
                if (utils_1.isTimeoutError(e)) {
                    debug(`ETIMEDOUT retring after 3 sec...`);
                    yield utils_1.delay(1000 * 3);
                    wikiEntities = yield wiki_entity_1.getEntities(findOptions);
                }
            }
            wikiEntities = wikiEntities || [];
            wikiEntities = wikiEntities.filter(item => !!item);
            wikiEntities = utils_1.uniqByProp(wikiEntities, 'id');
            if (wikiEntities.length === 0) {
                debug(`NO wiki entities found for ${titles.join('|')}`);
                return [];
            }
            else {
                debug(`Found wiki entities for ${titles.join('|')}: ${wikiEntities.map(item => item.label)}`);
            }
            let entities = wikiEntities.map(wikiEntity => this.wikiEntityBuilder.build({ wikiEntity }));
            entities = entities.filter(item => !wiki_entity_helper_1.WikiEntityHelper.isDisambiguation(item));
            return entities;
        });
    }
}
exports.FindWikiEntitiesByTitles = FindWikiEntitiesByTitles;
