"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug = require('debug')('textactor-explorer');
const entity_finder_1 = require("entity-finder");
const utils_1 = require("../../utils");
const usecase_1 = require("../usecase");
class FindWikiTitles extends usecase_1.UseCase {
    constructor(locale, countryTags) {
        super();
        this.locale = locale;
        this.tags = countryTags.getTags(locale.country, locale.lang);
    }
    innerExecute(names) {
        return __awaiter(this, void 0, void 0, function* () {
            names = utils_1.uniq(names);
            let allTitles = [];
            const tags = this.tags;
            for (let name of names) {
                debug(`finding wiki titles for ${name}...`);
                let titles;
                const findOptions = { limit: 5, tags, orderByTagsLimit: 2 };
                try {
                    titles = yield entity_finder_1.findTitles(name, this.locale.lang, findOptions);
                }
                catch (e) {
                    if (utils_1.isTimeoutError(e)) {
                        debug(`ETIMEDOUT retring after 3 sec...`);
                        yield utils_1.delay(1000 * 3);
                        titles = yield entity_finder_1.findTitles(name, this.locale.lang, findOptions);
                    }
                    else {
                        throw e;
                    }
                }
                if (titles && titles.length) {
                    const titleNames = titles.map(item => item.title).filter(item => item && item.trim().length >= 2);
                    debug(`found wiki titles for ${name}: ${titleNames}`);
                    allTitles = allTitles.concat(titleNames);
                }
                else {
                    debug(`NOT found wiki titles for ${name}`);
                }
                if (names.length > 1) {
                    yield utils_1.delay(500 * 1);
                }
            }
            return utils_1.uniq(allTitles);
        });
    }
}
exports.FindWikiTitles = FindWikiTitles;
