"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug = require('debug')('textactor-explorer');
const build_actor_by_names_1 = require("./build-actor-by-names");
const usecase_1 = require("../usecase");
class GenerateActors extends usecase_1.UseCase {
    constructor(locale, namesEnumerator, postNamesProcess, wikiEntityRep) {
        super();
        this.namesEnumerator = namesEnumerator;
        this.postNamesProcess = postNamesProcess;
        this.buildActor = new build_actor_by_names_1.BuildActorByNames(locale, wikiEntityRep);
    }
    innerExecute(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            let actor;
            while (!this.namesEnumerator.atEnd()) {
                try {
                    const actorNames = yield this.namesEnumerator.next();
                    if (!actorNames.length) {
                        debug(`GenerateActors: no names!`);
                        continue;
                    }
                    const names = actorNames.nameList();
                    actor = yield this.buildActor.execute(actorNames);
                    if (this.postNamesProcess) {
                        yield this.postNamesProcess.execute(names);
                    }
                    if (actor) {
                        if (this.postNamesProcess) {
                            yield this.postNamesProcess.execute(actor.names.map(item => item.name));
                        }
                    }
                }
                catch (e) {
                    return Promise.reject(e);
                }
                if (callback && actor) {
                    yield callback(actor);
                }
            }
        });
    }
}
exports.GenerateActors = GenerateActors;
