"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug = require('debug')('textactor-explorer');
const concept_helper_1 = require("../../entities/concept-helper");
const usecase_1 = require("../usecase");
const same_names_1 = require("same-names");
const utils_1 = require("../../utils");
class PushContextConcepts extends usecase_1.UseCase {
    constructor(conceptRep, knownNames) {
        super();
        this.conceptRep = conceptRep;
        this.knownNames = knownNames;
    }
    innerExecute(concepts) {
        concepts = concepts.filter(concept => concept_helper_1.ConceptHelper.isValid(concept));
        setSameIds(concepts);
        return Promise.all(concepts.map(concept => this.pushConcept(concept)));
    }
    pushConcept(concept) {
        return __awaiter(this, void 0, void 0, function* () {
            concept_helper_1.ConceptHelper.setRootIds(concept);
            const knownName = this.knownNames.getKnownName(concept.name, concept.lang, concept.country);
            if (knownName && knownName.name) {
                concept.knownName = knownName.name;
                debug(`set concept known name: ${concept.name}=>${concept.knownName}`);
            }
            return yield this.conceptRep.createOrUpdate(concept);
        });
    }
}
exports.PushContextConcepts = PushContextConcepts;
function setSameIds(concepts) {
    concepts = concepts.filter(concept => !concept.isAbbr && concept.nameLength > 4);
    const names = concepts.map(concept => concept.name)
        .concat(concepts.filter(concept => !!concept.knownName).map(concept => concept.knownName));
    for (let concept of concepts) {
        let sameNames = same_names_1.default(concept.name, names, { lang: concept.lang });
        if (sameNames && sameNames.length) {
            sameNames = sameNames.filter(item => item.name !== concept.name && item.rating > 0.5);
            if (concept.countWords === 1) {
                sameNames = sameNames.filter(item => item.rating > 0.7);
            }
            const sameIds = sameNames.map(item => concept_helper_1.ConceptHelper.rootId(item.name, concept.lang, concept.country, concept.containerId));
            concept.rootNameIds = concept.rootNameIds.concat(sameIds);
        }
        concept.rootNameIds = utils_1.uniq(concept.rootNameIds);
    }
}
