"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug = require('debug')('textactor-explorer');
const wiki_entity_helper_1 = require("../../entities/wiki-entity-helper");
const utils_1 = require("../../utils");
const usecase_1 = require("../usecase");
class SelectWikiEntity extends usecase_1.UseCase {
    constructor(locale, wikiEntityRepository) {
        super();
        this.locale = locale;
        this.wikiEntityRepository = wikiEntityRepository;
    }
    innerExecute(names) {
        return __awaiter(this, void 0, void 0, function* () {
            const nameHashes = wiki_entity_helper_1.WikiEntityHelper.namesHashes(names, this.locale.lang);
            let entities = [];
            for (let nameHash of nameHashes) {
                const list = yield this.wikiEntityRepository.getByNameHash(nameHash);
                entities = entities.concat(list);
            }
            if (entities.length) {
                debug(`Found wikientity by names: ${JSON.stringify(names)}`);
            }
            else {
                debug(`NOT Found wikientity by names: ${JSON.stringify(names)}`);
            }
            if (entities.length === 0 || this.countryWikiEntities(entities).length === 0) {
                let entitiesByPartialNames = [];
                for (let nameHash of nameHashes) {
                    const list = yield this.wikiEntityRepository.getByPartialNameHash(nameHash);
                    entitiesByPartialNames = entitiesByPartialNames.concat(list);
                }
                entitiesByPartialNames = this.countryWikiEntities(entitiesByPartialNames);
                if (entitiesByPartialNames.length) {
                    debug(`found locale entities by partial name: ${entitiesByPartialNames.map(item => item.name)}`);
                    entities = entities.concat(entitiesByPartialNames);
                }
                if (entities.length === 0) {
                    debug(`NOT Found wikientity by partial names: ${JSON.stringify(names)}`);
                    return null;
                }
            }
            entities = utils_1.uniqByProp(entities, 'id');
            entities = this.sortWikiEntities(entities);
            const entity = entities[0];
            return entity;
        });
    }
    sortWikiEntities(entities) {
        if (!entities.length) {
            return entities;
        }
        entities = sortEntities(entities);
        const topEntity = entities[0];
        if (topEntity.countryCodes && topEntity.countryCodes.indexOf(this.locale.country) > -1) {
            debug(`Top entity has country=${this.locale.country}: ${topEntity.name}`);
            return utils_1.uniqByProp(entities, 'id');
        }
        const countryEntities = this.countryWikiEntities(entities);
        if (countryEntities.length) {
            let useCountryEntity = false;
            const topEntityPopularity = wiki_entity_helper_1.WikiEntityHelper.getPopularity(topEntity.rank);
            const countryEntityPopularity = wiki_entity_helper_1.WikiEntityHelper.getPopularity(countryEntities[0].rank);
            if (topEntityPopularity < wiki_entity_helper_1.EntityPopularity.POPULAR || countryEntityPopularity > wiki_entity_helper_1.EntityPopularity.LOW) {
                useCountryEntity = true;
            }
            else {
                debug(`using POPULAR entity: ${topEntity.name}(${topEntity.rank}-${topEntityPopularity})>${countryEntities[0].rank}-(${countryEntityPopularity})`);
            }
            if (useCountryEntity) {
                entities = countryEntities.concat(entities);
                debug(`using country entity: ${entities[0].name}`);
            }
        }
        entities = utils_1.uniqByProp(entities, 'id');
        return entities;
    }
    countryWikiEntities(entities) {
        if (!entities.length) {
            return entities;
        }
        return entities.filter(item => item.countryCodes && item.countryCodes.indexOf(this.locale.country) > -1);
    }
}
exports.SelectWikiEntity = SelectWikiEntity;
function sortEntities(entities) {
    if (!entities.length) {
        return entities;
    }
    entities = entities.sort((a, b) => b.rank - a.rank);
    return entities;
}
