"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug = require('debug')('textactor-explorer');
const usecase_1 = require("./usecase");
const generate_actors_1 = require("./actions/generate-actors");
const delete_unpopular_concepts_1 = require("./actions/delete-unpopular-concepts");
const explore_wiki_entities_1 = require("./actions/explore-wiki-entities");
const delete_invalid_concepts_1 = require("./actions/delete-invalid-concepts");
const concept_container_1 = require("../entities/concept-container");
const concept_container_helper_1 = require("../entities/concept-container-helper");
const popular_concept_names_enumerator_1 = require("../services/popular-concept-names-enumerator");
const delete_actor_concepts_1 = require("./actions/delete-actor-concepts");
const clean_concept_container_1 = require("./actions/clean-concept-container");
class ExploreContainer extends usecase_1.UseCase {
    constructor(container, containerRep, conceptRep, entityRep, wikiSearchNameRep, wikiTitleRep, countryTags, knownNames) {
        super();
        this.container = container;
        this.containerRep = containerRep;
        this.conceptRep = conceptRep;
        this.entityRep = entityRep;
        this.wikiSearchNameRep = wikiSearchNameRep;
        this.wikiTitleRep = wikiTitleRep;
        this.countryTags = countryTags;
        this.knownNames = knownNames;
        if (!container.lang || !container.country) {
            throw new Error(`ConceptContainer is not valid: ${container.lang}-${container.country}`);
        }
    }
    innerExecute(callback, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const container = this.container;
            debug(`Start processing concepts... ${JSON.stringify(options)}`);
            if (!concept_container_helper_1.ConceptContainerHelper.canStartGenerate(container.status)) {
                return Promise.reject(new Error(`ConceptContainer is not generateable: ${container.status}`));
            }
            const deleteUnpopularConcepts = new delete_unpopular_concepts_1.DeleteUnpopularConcepts(container, this.conceptRep);
            const deleteInvalidConcepts = new delete_invalid_concepts_1.DeleteInvalidConcepts(container, this.conceptRep, this.entityRep);
            const exploreWikiEntities = new explore_wiki_entities_1.ExploreWikiEntities(container, new popular_concept_names_enumerator_1.PopularConceptNamesEnumerator({ mutable: false }, container, this.conceptRep), this.entityRep, this.wikiSearchNameRep, this.wikiTitleRep, this.countryTags, this.knownNames);
            const generateActors = new generate_actors_1.GenerateActors(this.container, new popular_concept_names_enumerator_1.PopularConceptNamesEnumerator({ mutable: true }, container, this.conceptRep), new delete_actor_concepts_1.DeleteActorConcepts(container, this.conceptRep), this.entityRep);
            const cleanContainer = new clean_concept_container_1.CleanConceptContainer(this.conceptRep);
            yield this.containerRep.update({
                id: this.container.id,
                set: {
                    status: concept_container_1.ConceptContainerStatus.GENERATING
                }
            });
            try {
                debug(`<===== Start deleteInvalidConcepts`);
                yield deleteInvalidConcepts.execute(undefined);
                debug(`<===== End deleteInvalidConcepts`);
                debug(`=====> Start deleteUnpopularConcepts`);
                yield deleteUnpopularConcepts.execute(options);
                debug(`<===== End deleteUnpopularConcepts`);
                debug(`=====> Start exploreWikiEntities`);
                yield exploreWikiEntities.execute(undefined);
                debug(`<===== End exploreWikiEntities`);
                debug(`<===== Start deleteInvalidConcepts`);
                yield deleteInvalidConcepts.execute(undefined);
                debug(`<===== End deleteInvalidConcepts`);
                debug(`=====> Start generateActors`);
                yield generateActors.execute(callback);
                debug(`<===== End generateActors`);
                yield cleanContainer.execute(container);
            }
            catch (e) {
                const error = e.message;
                yield this.containerRep.update({
                    id: this.container.id,
                    set: {
                        status: concept_container_1.ConceptContainerStatus.GENERATE_ERROR,
                        lastError: error
                    }
                });
                throw e;
            }
            yield this.containerRep.update({
                id: this.container.id,
                set: {
                    status: concept_container_1.ConceptContainerStatus.EMPTY
                }
            });
        });
    }
}
exports.ExploreContainer = ExploreContainer;
