"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug = require('debug')('textactor-explorer');
const usecase_1 = require("./usecase");
const explore_wiki_entities_by_names_1 = require("./actions/explore-wiki-entities-by-names");
const build_actor_by_names_1 = require("./actions/build-actor-by-names");
const actor_name_collection_1 = require("../entities/actor-name-collection");
class ExploreName extends usecase_1.UseCase {
    constructor(locale, entityRep, wikiSearchNameRep, wikiTitleRep, countryTags, knownNames) {
        super();
        this.locale = locale;
        this.entityRep = entityRep;
        this.wikiSearchNameRep = wikiSearchNameRep;
        this.wikiTitleRep = wikiTitleRep;
        this.countryTags = countryTags;
        this.knownNames = knownNames;
        if (!locale.lang || !locale.country) {
            throw new Error(`Locale is not valid: ${locale.lang}-${locale.country}`);
        }
        this.actorBuilder = new build_actor_by_names_1.BuildActorByNames(locale, entityRep);
        this.exploreWikiEntities = new explore_wiki_entities_by_names_1.ExploreWikiEntitiesByNames(locale, this.entityRep, this.wikiSearchNameRep, this.wikiTitleRep, this.countryTags, this.knownNames);
    }
    innerExecute(name) {
        return __awaiter(this, void 0, void 0, function* () {
            name = Array.isArray(name) ? name : [name];
            const lang = this.locale.lang;
            debug(`Start processing: ${name}`);
            debug(`=====> Start exploreWikiEntities`);
            yield this.exploreWikiEntities.execute(name);
            debug(`<===== End exploreWikiEntities`);
            debug(`=====> Start generateActors`);
            const actor = yield this.actorBuilder.execute(actor_name_collection_1.ActorNameCollection.fromArray(name, lang));
            debug(`<===== End generateActors`);
            debug(`End processing name: ${name}`);
            return actor;
        });
    }
}
exports.ExploreName = ExploreName;
