"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const crypto = require("crypto");
function uniqByProp(items, prop) {
    const map = {};
    const list = [];
    for (let item of items) {
        if (map[item[prop]] === undefined) {
            map[item[prop]] = 1;
            list.push(item);
        }
    }
    return list;
}
exports.uniqByProp = uniqByProp;
function isTimeoutError(error) {
    return error && error.code && ['ESOCKETTIMEDOUT', 'ETIMEDOUT'].indexOf(error.code) > -1;
}
exports.isTimeoutError = isTimeoutError;
function md5(value) {
    return crypto.createHash('md5').update(value, 'utf8').digest('hex').toLowerCase();
}
exports.md5 = md5;
function uniq(items) {
    return items.filter((value, index, self) => self.indexOf(value) === index);
}
exports.uniq = uniq;
function mapPromise(keys, callback) {
    const tasks = keys.map(key => callback(key).then(result => ({ key, result })));
    return Promise.all(tasks)
        .then(results => {
        const response = new Map();
        for (let item of results) {
            response.set(item.key, item.result);
        }
        return response;
    });
}
exports.mapPromise = mapPromise;
exports.LANG_REG = /^[a-z]{2}$/;
exports.COUNTRY_REG = /^[a-z]{2}$/;
function unixTimestamp() {
    return Math.round(Date.now() / 1000);
}
exports.unixTimestamp = unixTimestamp;
function delay(ms) {
    return new Promise(resolve => setTimeout(resolve, ms));
}
exports.delay = delay;
function filterStrings(str) {
    return (str && str.filter(item => item && item.trim().length > 0) || []);
}
exports.filterStrings = filterStrings;
